/**********************************************************************
Copyright (c) 2006 Thomas Marti, Stefan Schmid and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.datanucleus.store.exception;

import org.datanucleus.exceptions.NucleusDataStoreException;

/**
 * A <tt>UnsupportedMethodException</tt> is thrown if an attempt is made
 * to call a method in a query that is not supported by the database.
 *
 * @version $Revision: 1.1 $ 
 */
public class UnsupportedMethodException extends NucleusDataStoreException
{
    /**
     * Constructs an unsupported method exception with no specific detail
     * message.
     */
    public UnsupportedMethodException()
    {
        super();
        setFatal();
    }

    /**
     * Constructs an unsupported method exception with the specified detail
     * message.
     *
     * @param msg       the detail message
     */
    public UnsupportedMethodException(String msg)
    {
        super(msg);
        setFatal();
    }

    /**
     * Constructs an unsupported method exception with the specified detail
     * message and nested exception.
     *
     * @param msg       the detail message
     * @param nested    the nested exception(s).
     */
    public UnsupportedMethodException(String msg, Exception nested)
    {
        super(msg, nested);
        setFatal();
    }
}