/**********************************************************************
Copyright (c) 2006 Thomas Marti, Stefan Schmid and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
2006 Erik Bengtson - Original SpatialExpression implementation
    ...
**********************************************************************/
package org.datanucleus.store.expression.spatial;

import java.util.List;

import org.datanucleus.store.mapped.expression.BooleanExpression;
import org.datanucleus.store.mapped.expression.LogicSetExpression;
import org.datanucleus.store.mapped.expression.NullLiteral;
import org.datanucleus.store.mapped.expression.QueryExpression;
import org.datanucleus.store.mapped.expression.ScalarExpression;
import org.datanucleus.store.mapped.mapping.JavaTypeMapping;

/**
 * Representation of an expression with a geometry.
 * @version $Revision: 1.1 $
 */

public class GeometryScalarExpression extends ScalarExpression
{

    public GeometryScalarExpression(QueryExpression qs)
    {
        super(qs);
    }

    public GeometryScalarExpression(QueryExpression qs, JavaTypeMapping mapping, LogicSetExpression te)
    {
        super(qs, mapping, te);
    }

    public GeometryScalarExpression(String functionName, List args)
    {
        super(functionName, args);
    }

    public GeometryScalarExpression(String functionName, List args, JavaTypeMapping mapping)
    {
        super(functionName, args);
        this.mapping = mapping;
    }

    public GeometryScalarExpression(String functionName, List args, List types, JavaTypeMapping mapping)
    {
        super(functionName, args, types);
        this.mapping = mapping;
    }

    public BooleanExpression eq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.eq(this);
        }
        else if (expr instanceof GeometryLiteral)
        {
            return new BooleanExpression(this, OP_EQ, expr);
        }
        else if (expr instanceof GeometryScalarExpression)
        {
            return new BooleanExpression(this, OP_EQ, expr);
        }
        else
        {
            return super.eq(expr);
        }
    }

    public BooleanExpression noteq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.noteq(this);
        }
        else if (expr instanceof GeometryLiteral)
        {
            return new BooleanExpression(this, OP_NOTEQ, expr);
        }
        else if (expr instanceof GeometryScalarExpression)
        {
            return new BooleanExpression(this, OP_NOTEQ, expr);
        }
        else
        {
            return super.noteq(expr);
        }
    }

}
