/**********************************************************************
 Copyright (c) 2006 Thomas Marti, Stefan Schmid and others. All rights reserved.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 Contributors:
 ...
 **********************************************************************/
package org.datanucleus.store.expression.spatial;

import org.datanucleus.store.exception.UnsupportedMethodException;
import org.datanucleus.store.mapped.expression.QueryExpression;
import org.datanucleus.store.mapped.expression.ScalarExpression;
import org.datanucleus.store.mapped.mapping.JavaTypeMapping;

/**
 * Spatial expressions representations for MySQL.
 * @version $Revision: 1.1 $
 */

public class MysqlSpatialExpression extends SpatialExpression
{

    public MysqlSpatialExpression(QueryExpression qs)
    {
        super(qs);
    }

    // ---- OGC SF functions ----//

    public ScalarExpression isEmptyMethod(ScalarExpression geom)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression isSimpleMethod(ScalarExpression geom)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression boundaryMethod(ScalarExpression geom)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression crossesMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression relateMethod(ScalarExpression geom1, ScalarExpression geom2, ScalarExpression patternMatrix)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression distanceMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression bufferMethod(ScalarExpression geom, ScalarExpression distance)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression convexHullMethod(ScalarExpression geom)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression intersectionMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression unionMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression differenceMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression symDifferenceMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression isRingMethod(ScalarExpression geom)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression lengthMethod(ScalarExpression geom)
    {
        JavaTypeMapping doubleMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Double.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("GLength", geom, doubleMapping);
    }

    public ScalarExpression centroidMethod(ScalarExpression geom)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression pointOnSurfaceMethod(ScalarExpression geom)
    {
        throw new UnsupportedMethodException("MySQL does not yet (Version 5.1) implement this OGC SF function");
    }

    public ScalarExpression numInteriorRingMethod(ScalarExpression geom)
    {
        JavaTypeMapping integerMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Integer.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("NumInteriorRings", geom, integerMapping);
    }

    // ---- bounding box ----//

    public ScalarExpression bboxTestMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("MBRIntersects", geom1, geom2);
    }

    // ---- MySQL ----//

    public ScalarExpression mbrEqualMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("MBREqual", geom1, geom2);
    }

    public ScalarExpression mbrDisjointMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("MBRDisjoint", geom1, geom2);
    }

    public ScalarExpression mbrIntersectsMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("MBRIntersects", geom1, geom2);
    }

    public ScalarExpression mbrTouchesMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("MBRTouches", geom1, geom2);
    }

    public ScalarExpression mbrWithinMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("MBRWithin", geom1, geom2);
    }

    public ScalarExpression mbrContainsMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("MBRContains", geom1, geom2);
    }

    public ScalarExpression mbrOverlapsMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("MBROverlaps", geom1, geom2);
    }

}