/**********************************************************************
 Copyright (c) 2006 Thomas Marti, Stefan Schmid and others. All rights reserved.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 Contributors:
 2006 Erik Bengtson - Original SpatialExpression implementation
 ...
 **********************************************************************/

package org.datanucleus.store.expression.spatial;

import java.util.ArrayList;

import org.datanucleus.store.mapped.expression.BinaryExpression;
import org.datanucleus.store.mapped.expression.QueryExpression;
import org.datanucleus.store.mapped.expression.ScalarExpression;
import org.datanucleus.store.mapped.expression.StringExpression;
import org.datanucleus.store.mapped.expression.StringLiteral;
import org.datanucleus.store.mapped.mapping.JavaTypeMapping;

/**
 * Spatial expressions representations for Oracle.
 */
public class OracleSpatialExpression extends SpatialExpression
{
    public OracleSpatialExpression(QueryExpression qs)
    {
        super(qs);
    }

    // ---- OGC SF functions for constructing a geometry value given its Well-known Text Representation ----//

    public ScalarExpression geomFromTextMethod(ScalarExpression wkt, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkt, srid, null);
    }

    public ScalarExpression pointFromTextMethod(ScalarExpression wkt, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkt, srid, null);
    }

    public ScalarExpression lineFromTextMethod(ScalarExpression wkt, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkt, srid, null);
    }

    public ScalarExpression polyFromTextMethod(ScalarExpression wkt, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkt, srid, null);
    }

    public ScalarExpression mPointFromTextMethod(ScalarExpression wkt, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkt, srid, null);
    }

    public ScalarExpression mLineFromTextMethod(ScalarExpression wkt, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkt, srid, null);
    }

    public ScalarExpression mPolyFromTextMethod(ScalarExpression wkt, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkt, srid, null);
    }

    public ScalarExpression geomCollFromTextMethod(ScalarExpression wkt, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkt, srid, null);
    }

    // ---- OGC SF functions for constructing a geometric object given its Well-known Binary Representation ----//

    public ScalarExpression geomFromWKBMethod(ScalarExpression wkb, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkb, srid, null);
    }

    public ScalarExpression pointFromWKBMethod(ScalarExpression wkb, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkb, srid, null);
    }

    public ScalarExpression lineFromWKBMethod(ScalarExpression wkb, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkb, srid, null);
    }

    public ScalarExpression polyFromWKBMethod(ScalarExpression wkb, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkb, srid, null);
    }

    public ScalarExpression mPointFromWKBMethod(ScalarExpression wkb, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkb, srid, null);
    }

    public ScalarExpression mLineFromWKBMethod(ScalarExpression wkb, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkb, srid, null);
    }

    public ScalarExpression mPolyFromWKBMethod(ScalarExpression wkb, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkb, srid, null);
    }

    public ScalarExpression geomCollFromWKBMethod(ScalarExpression wkb, ScalarExpression srid)
    {
        return geometryFunction("SDO_GEOMETRY", wkb, srid, null);
    }

    // ---- OGC SF basic methods on geometric objects ----//

    public ScalarExpression dimensionMethod(ScalarExpression geom)
    {
        JavaTypeMapping integerMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Integer.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("dimension", geometryFunction("geometry.from_sdo_geom", geom, getGeomMapping(geom)), integerMapping);
    }

    public ScalarExpression geometryTypeMethod(ScalarExpression geom)
    {
        return stringFunction("geometryType", geometryFunction("geometry.from_sdo_geom", geom, getGeomMapping(geom)));
    }

    public ScalarExpression sridMethod(ScalarExpression geom)
    {
        JavaTypeMapping integerMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Integer.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("srid", geometryFunction("geometry.from_sdo_geom", geom, getGeomMapping(geom)), integerMapping);
    }

    public ScalarExpression envelopeMethod(ScalarExpression geom)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("envelope",
            geometryFunction("geometry.from_sdo_geom", geom, null), null), getGeomMapping(geom));
    }

    public ScalarExpression asTextMethod(ScalarExpression geom)
    {
        return stringFunction("asText", geometryFunction("geometry.from_sdo_geom", geom, getGeomMapping(geom)));
    }

    public ScalarExpression asBinaryMethod(ScalarExpression geom)
    {
        ArrayList args = new ArrayList();
        args.add(geometryFunction("geometry.from_sdo_geom", geom, getGeomMapping(geom)));
        return new BinaryExpression("asBinary", args);
    }

    public ScalarExpression isEmptyMethod(ScalarExpression geom)
    {
        return booleanFunction("isEmpty", geometryFunction("geometry.from_sdo_geom", geom, getGeomMapping(geom)));
    }

    public ScalarExpression isSimpleMethod(ScalarExpression geom)
    {
        return booleanFunction("isSimple", geometryFunction("geometry.from_sdo_geom", geom, getGeomMapping(geom)));
    }

    public ScalarExpression boundaryMethod(ScalarExpression geom)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("boundary",
            geometryFunction("geometry.from_sdo_geom", geom, null), null), getGeomMapping(geom));
    }

    // ---- OGC SF methods for testing spatial relations between geometric objects ----//

    public ScalarExpression equalsMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("equals", geometryFunction("geometry.from_sdo_geom", geom1, null), geometryFunction(
            "geometry.from_sdo_geom", geom2, null));
    }

    public ScalarExpression disjointMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("disjoint", geometryFunction("geometry.from_sdo_geom", geom1, null), geometryFunction(
            "geometry.from_sdo_geom", geom2, null));
    }

    public ScalarExpression intersectsMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("intersects", geometryFunction("geometry.from_sdo_geom", geom1, null), geometryFunction(
            "geometry.from_sdo_geom", geom2, null));
    }

    public ScalarExpression touchesMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("touch", geometryFunction("geometry.from_sdo_geom", geom1, null), geometryFunction("geometry.from_sdo_geom",
            geom2, null));
    }

    public ScalarExpression crossesMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("cross", geometryFunction("geometry.from_sdo_geom", geom1, null), geometryFunction("geometry.from_sdo_geom",
            geom2, null));
    }

    public ScalarExpression withinMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("within", geometryFunction("geometry.from_sdo_geom", geom1, null), geometryFunction(
            "geometry.from_sdo_geom", geom2, null));
    }

    public ScalarExpression containsMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("ogc_contains", geometryFunction("geometry.from_sdo_geom", geom1, null), geometryFunction(
            "geometry.from_sdo_geom", geom2, null));
    }

    public ScalarExpression overlapsMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return booleanFunction("overlap", geometryFunction("geometry.from_sdo_geom", geom1, null), geometryFunction(
            "geometry.from_sdo_geom", geom2, null));
    }

    public ScalarExpression relateMethod(ScalarExpression geom1, ScalarExpression geom2, ScalarExpression patternMatrix)
    {
        return booleanFunction("relate", geometryFunction("geometry.from_sdo_geom", geom1, null), geometryFunction(
            "geometry.from_sdo_geom", geom2, null), patternMatrix);
    }

    // ---- OGC SF methods that support spatial analysis ----//

    public ScalarExpression distanceMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        JavaTypeMapping doubleMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Double.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("distance", geometryFunction("geometry.from_sdo_geom", geom1, null), geometryFunction(
            "geometry.from_sdo_geom", geom2, null), doubleMapping);
    }

    public ScalarExpression bufferMethod(ScalarExpression geom, ScalarExpression distance)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("buffer", geometryFunction("geometry.from_sdo_geom", geom, null),
            distance, null), getGeomMapping(geom));
    }

    public ScalarExpression convexHullMethod(ScalarExpression geom)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("convexHull", geometryFunction("geometry.from_sdo_geom", geom,
            null), null), getGeomMapping(geom));
    }

    public ScalarExpression intersectionMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("intersection", geometryFunction("geometry.from_sdo_geom", geom1,
            null), geometryFunction("geometry.from_sdo_geom", geom2, null), null), getGeomMapping(geom1));
    }

    public ScalarExpression unionMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("ogc_union", geometryFunction("geometry.from_sdo_geom", geom1,
            null), geometryFunction("geometry.from_sdo_geom", geom2, null), null), getGeomMapping(geom1));
    }

    public ScalarExpression differenceMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("difference", geometryFunction("geometry.from_sdo_geom", geom1,
            null), geometryFunction("geometry.from_sdo_geom", geom2, null), null), getGeomMapping(geom1));
    }

    public ScalarExpression symDifferenceMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("symmetricDifference", geometryFunction("geometry.from_sdo_geom",
            geom1, null), geometryFunction("geometry.from_sdo_geom", geom2, null), null), getGeomMapping(geom1));
    }

    // ---- OGC SF methods on on type Point ----//

    public ScalarExpression xMethod(ScalarExpression geom)
    {
        JavaTypeMapping doubleMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Double.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("x", geometryFunction("treat", geometryFunction("geometry.from_sdo_geom", geom, null), "point", null),
            doubleMapping);
    }

    public ScalarExpression yMethod(ScalarExpression geom)
    {
        JavaTypeMapping doubleMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Double.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("y", geometryFunction("treat", geometryFunction("geometry.from_sdo_geom", geom, null), "point", null),
            doubleMapping);
    }

    // ---- OGC SF methods on type Curve ----//

    public ScalarExpression startPointMethod(ScalarExpression geom)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("startPoint", geometryFunction("treat", geometryFunction(
            "geometry.from_sdo_geom", geom, null), "curve", null), null), getGeomMapping(geom));
    }

    public ScalarExpression endPointMethod(ScalarExpression geom)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("endPoint", geometryFunction("treat", geometryFunction(
            "geometry.from_sdo_geom", geom, null), "curve", null), null), getGeomMapping(geom));
    }

    public ScalarExpression isRingMethod(ScalarExpression geom)
    {
        return booleanFunction("isRing", geometryFunction("treat", geometryFunction("geometry.from_sdo_geom", geom, null), "curve", null));
    }

    // ---- OGC SF methods on type Curve and MultiCurve----//

    public ScalarExpression isClosedMethod(ScalarExpression geom)
    {
        return booleanFunction("isClosed", geometryFunction("treat", geometryFunction("geometry.from_sdo_geom", geom, null), "curve", null));
    }

    public ScalarExpression lengthMethod(ScalarExpression geom)
    {
        JavaTypeMapping doubleMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Double.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("ogc_length", geometryFunction("treat", geometryFunction("geometry.from_sdo_geom", geom, null), "curve",
            null), doubleMapping);
    }

    // ---- OGC SF methods on type LineString ----//

    public ScalarExpression numPointsMethod(ScalarExpression geom)
    {
        JavaTypeMapping integerMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Integer.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("numPoints", geometryFunction("treat", geometryFunction("geometry.from_sdo_geom", geom, null), "LineString",
            null), integerMapping);
    }

    public ScalarExpression pointNMethod(ScalarExpression geom, ScalarExpression n)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("pointN", geometryFunction("treat", geometryFunction(
            "geometry.from_sdo_geom", geom, null), "LineString", null), n, null), getGeomMapping(geom));
    }

    // ---- OGC SF methods on type Surface and MultiSurface ----//

    public ScalarExpression areaMethod(ScalarExpression geom)
    {
        JavaTypeMapping doubleMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Double.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("area", geometryFunction("treat", geometryFunction("geometry.from_sdo_geom", geom, null), "surface", null),
            doubleMapping);
    }

    public ScalarExpression centroidMethod(ScalarExpression geom)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("centroid", geometryFunction("treat", geometryFunction(
            "geometry.from_sdo_geom", geom, null), "surface", null), null), getGeomMapping(geom));
    }

    public ScalarExpression pointOnSurfaceMethod(ScalarExpression geom)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("pointOnSurface", geometryFunction("treat", geometryFunction(
            "geometry.from_sdo_geom", geom, null), "surface", null), null), getGeomMapping(geom));
    }

    // ---- OGC SF methods on type Polygon ----//

    public ScalarExpression exteriorRingMethod(ScalarExpression geom)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("exteriorRing", geometryFunction("treat", geometryFunction(
            "geometry.from_sdo_geom", geom, null), "Polygon", null), null), getGeomMapping(geom));
    }

    public ScalarExpression numInteriorRingMethod(ScalarExpression geom)
    {
        JavaTypeMapping integerMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Integer.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("numInteriorRings", geometryFunction("treat", geometryFunction("geometry.from_sdo_geom", geom, null),
            "Polygon", null), integerMapping);
    }

    public ScalarExpression interiorRingNMethod(ScalarExpression geom, ScalarExpression n)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("interiorRingN", geometryFunction("treat", geometryFunction(
            "geometry.from_sdo_geom", geom, null), "Polygon", null), n, null), getGeomMapping(geom));
    }

    // ---- OGC SF methods on type GeomCollection ----//

    public ScalarExpression numGeometriesMethod(ScalarExpression geom)
    {
        JavaTypeMapping integerMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
            Integer.class, false, false, qs.getClassLoaderResolver());
        return numericFunction("numGeometries", geometryFunction("treat", geometryFunction("geometry.from_sdo_geom", geom, null),
            "GeometryCollection", null), integerMapping);
    }

    public ScalarExpression geometryNMethod(ScalarExpression geom, ScalarExpression n)
    {
        return geometryFunction("geometry.get_sdo_geom", geometryFunction("geometryN", geometryFunction("treat", geometryFunction(
            "geometry.from_sdo_geom", geom, null), "GeometryCollection", null), n, null), getGeomMapping(geom));
    }

    // ---- bounding box ----//

    public ScalarExpression bboxTestMethod(ScalarExpression geom1, ScalarExpression geom2)
    {
        return oracleBooleanFunction("SDO_FILTER", geom1, geom2);
    }

    // ---- Oracle ---- //

    public ScalarExpression sdo_geometryMethod(ScalarExpression gtype, ScalarExpression srid, ScalarExpression point,
            ScalarExpression elem_info, ScalarExpression ordinates)
    {
        return geometryFunction("SDO_GEOMETRY", gtype, srid, point, elem_info, ordinates, null);
    }

    public ScalarExpression sdo_point_typeMethod(ScalarExpression x, ScalarExpression y, ScalarExpression z)
    {
        return geometryFunction("SDO_POINT_TYPE", x, y, z, null);
    }

    public ScalarExpression sdo_elem_info_arrayMethod(ScalarExpression numbers)
    {
        ArrayList args = new ArrayList();
        if (numbers instanceof StringLiteral)
        {
            JavaTypeMapping integerMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
                Integer.class, false, false, qs.getClassLoaderResolver());
            String[] token = ((String) ((StringLiteral) numbers).getValue()).split(",");
            try
            {
                for (int i = 0; i < token.length; i++)
                {
                    args.add(integerMapping.newLiteral(qs, Integer.valueOf(token[i])));
                }
            }
            catch (NumberFormatException e)
            {
                throw new IllegalArgumentTypeException(numbers);
            }
        }
        if (args.isEmpty())
            throw new IllegalArgumentTypeException(numbers);
        return new GeometryScalarExpression("SDO_ELEM_INFO_ARRAY", args);
    }

    public ScalarExpression sdo_ordinate_arrayMethod(ScalarExpression numbers)
    {
        ArrayList args = new ArrayList();
        if (numbers instanceof StringLiteral)
        {
            JavaTypeMapping doubleMapping = qs.getStoreManager().getMappingManager().getMappingWithDatastoreMapping(
                Double.class, false, false, qs.getClassLoaderResolver());
            String[] token = ((String) ((StringLiteral) numbers).getValue()).split(",");
            try
            {
                for (int i = 0; i < token.length; i++)
                {
                    args.add(doubleMapping.newLiteral(qs, Double.valueOf(token[i])));
                }
            }
            catch (NumberFormatException e)
            {
                throw new IllegalArgumentTypeException(numbers);
            }
        }
        if (args.isEmpty())
            throw new IllegalArgumentTypeException(numbers);
        return new GeometryScalarExpression("SDO_ORDINATE_ARRAY", args);
    }

    // ---- ----//

    private ScalarExpression oracleBooleanFunction(String functionName, ScalarExpression expr1, ScalarExpression expr2)
    {
        ArrayList args = new ArrayList();
        args.add(expr1);
        args.add(expr2);
        return new SpatialBooleanExpression(new StringExpression(functionName, args), new StringLiteral(qs, null, "TRUE"));
    }
}