/**********************************************************************
 Copyright (c) 2006 Thomas Marti, Stefan Schmid and others. All rights reserved.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 Contributors:
 ...
 **********************************************************************/
package org.datanucleus.store.mapping.pg;

import java.sql.SQLException;

import org.datanucleus.ClassLoaderResolver;
import org.datanucleus.store.mapped.expression.LogicSetExpression;
import org.datanucleus.store.mapped.expression.ObjectExpression;
import org.datanucleus.store.mapped.expression.ObjectLiteral;
import org.datanucleus.store.mapped.expression.QueryExpression;
import org.datanucleus.store.mapped.expression.ScalarExpression;
import org.datanucleus.store.mapped.mapping.SingleFieldMapping;
import org.postgis.PGbox2d;

/**
 * Mapping for org.postgis.PGbox2d to its datastore representation.
 */
public class PGbox2dMapping extends SingleFieldMapping
{

    private static final PGbox2d sampleValue;
    static
    {
        try
        {
            sampleValue = new PGbox2d("(0 0, 1 1)");
        }
        catch (SQLException e)
        {
            throw new Error("AssertionError: Illegal sampleValue", e);
        }
    }

    public Class getJavaType()
    {
        return PGbox2d.class;
    }

    public Object getSampleValue(ClassLoaderResolver clr)
    {
        return sampleValue;
    }

    public ScalarExpression newLiteral(QueryExpression qs, Object value)
    {
        return new ObjectLiteral(qs, this, value);
    }

    public ScalarExpression newScalarExpression(QueryExpression qs, LogicSetExpression te)
    {
        return new ObjectExpression(qs, this, te);
    }

}
