/**********************************************************************
 Copyright (c) 2006 Thomas Marti, Stefan Schmid and others. All rights reserved.
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

 Contributors:
 ...
 **********************************************************************/
package org.datanucleus.store.rdbms.mapping.jts2oracle;

import org.datanucleus.store.mapped.DatastoreField;
import org.datanucleus.store.mapped.MappedStoreManager;
import org.datanucleus.store.mapped.mapping.JavaTypeMapping;

import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.LinearRing;

/**
 * Mapping for a JTS Linearring object to Oracle.
 * 
 * @version $Revision: 1.1 $
 */
public class LinearRingRDBMSMapping extends GeometryRDBMSMapping
{

    public LinearRingRDBMSMapping(JavaTypeMapping mapping, MappedStoreManager storeMgr, DatastoreField field)
    {
        super(mapping, storeMgr, field);
    }

    public LinearRingRDBMSMapping(MappedStoreManager storeMgr, JavaTypeMapping mapping)
    {
        super(storeMgr, mapping);
    }
    
    public Object getObject(Object rs, int exprIndex)
    {
        LineString lineString = (LineString)super.getObject(rs, exprIndex);
        if (lineString == null) return null;
        LinearRing linearRing = new LinearRing(lineString.getCoordinateSequence(), lineString.getFactory());
        linearRing.setSRID(lineString.getSRID());
        return linearRing;
    }
    
    public void setObject(Object preparedStatement, int exprIndex, Object value)
    {
        LineString lineString = null;
        if (value != null)
        {
            LinearRing linearRing = (LinearRing)value;
            lineString = new LineString(linearRing.getCoordinateSequence(), linearRing.getFactory());
            lineString.setSRID(linearRing.getSRID());
        }
        super.setObject(preparedStatement, exprIndex, lineString);
    }

}
